/* Measure string and memory functions.
   Copyright (C) 2013-2022 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */
#include <stddef.h>
#include <getopt.h>
#include <sys/cdefs.h>
//#include <programs/xmalloc.h>

#include "bench-timing.h"
#include <unistd.h>
#include <string.h>
#include <malloc.h>
#include <error.h>
#include <stdlib.h>

/* We are compiled under _ISOMAC, so libc-symbols.h does not do this
   for us.  */
//#include "config.h"
#ifdef HAVE_CC_INHIBIT_LOOP_TO_LIBCALL
# define inhibit_loop_to_libcall \
    __attribute__ ((__optimize__ ("-fno-tree-loop-distribute-patterns")))
#else
# define inhibit_loop_to_libcall
#endif



#ifndef TEST_MEMCMP
typedef struct
{
  const char *name;
  void (*fn) (void);
  long test;
} impl_t;
#else
typedef struct
{
  const char *name;
  int (*fn) (void);
  long test;
} impl_t;
#endif


int ret=0;

#ifdef TEST_MEMMOVE

extern void *__memmove_sve(void *d, const void *s, size_t n);
extern void *__memmove_simd(void *d, const void *s, size_t n);
extern void *__builtin_memmove(void *d, const void *s, size_t n);

impl_t impls[]={ 
//{"SVE", (void *)  __memmove_sve, 1},
//{"simd", (void *)  __memmove_simd, 1},
//{"sve", (void *)  __memmove_sve, 1},
//{"builtin", (void *)  __builtin_memmove, 1},
{"memmove", (void *)  memmove, 1},
{"", NULL, 0}
};
#endif

#ifdef TEST_MEMCMP

#define CHAR char
#define CHARBYTES 1
#define MAX_CHAR 255

impl_t impls[]={ 
//{"SVE", (void *)  __memmove_sve, 1},
//{"simd", (void *)  __memmove_simd, 1},
//{"sve", (void *)  __memmove_sve, 1},
//{"builtin", (void *)  __builtin_memmove, 1},
{"memcmp", (void *)  memcmp, 1},
{"", NULL, 0}
};
#endif



#ifdef TEST_MEMCPY

extern void *__memcpy_sve(void *d, const void *s, size_t n);
extern void *__memcpy_simd(void *d, const void *s, size_t n);
extern void *__builtin_memcpy(void *d, const void *s, size_t n);

impl_t impls[]={ 
{"memcpy", (void *)  memcpy, 1},
//{"SVE", (void *)  __memcpy_sve, 1},
//{"builtin", (void *)  __builtin_memcpy, 1},
//{"simd", (void *)  __memcpy_simd, 1},
{"", NULL, 0}
};
#endif



# define INNER_LOOP_ITERS 8192
# define INNER_LOOP_ITERS8 32768
# define INNER_LOOP_ITERS_LARGE 131072
# define INNER_LOOP_ITERS_MEDIUM 2048
# define INNER_LOOP_ITERS_SMALL 256

#define MIN_SIZE 32768
#define MAX_SIZE (1024*1024)
#define NUM_TESTS 16384

#ifndef MIN_PAGE_SIZE
#define MIN_PAGE_SIZE (MAX_SIZE + getpagesize())
#endif

# define CALL(impl, ...)	\
    (* (proto_t) (impl)->fn) (__VA_ARGS__)

#  define FOR_EACH_IMPL(impl, notall) \
    for(impl_t *impl=impls; impl->fn; impl++) \
       if (!notall || impl->test)

#define IMPL(a,b)

unsigned char *buf1, *buf2;
static size_t buf1_size, buf2_size, page_size;


static void
test_init (void)
{
  buf1 = malloc(MAX_SIZE);
  buf2 = malloc(MAX_SIZE);
  buf1_size = MAX_SIZE;
  buf2_size = MAX_SIZE;
  page_size = 8192;

  memset (buf1, 1, MAX_SIZE);
}


//#endif /* TEST_MAIN */
