#!/usr/bin/python3
import statistics
import sys
import json

label = "length"
# timings for one length are grouped in a list which is then add it to a dictionary that uses the length as key
def load(fname):
    r = {}
    global label
    label = "length"
    with open(fname) as fp:
        results = json.load(fp)['functions']
        if 'memmove' in results:
            if 'random' in results['memmove']['bench-variant']:
                label = "max src/dst offset"
            results = results['memmove']['results']
        elif 'memcpy' in results:
            if 'random' in results['memcpy']['bench-variant']:
                label = "max src/dst offset"
            results = results['memcpy']['results']
        elif 'memcmp' in results:
            results = results['memcmp']['results']
        else:
            Exception('could not find any of the known keys (memcpy, memcmp, memmove) in the json file')

    for result in results:
        time = result['timings']
        length = result['length']
        if length not in r:
            r[length]=[]
        r[length].append(float(time[0]))

    return r

#load all the runs into a new dict, as above: "128 : [<mean of the timings for length 128 of run 1>, <mean of the timings for length 128 of run 2>, .."
# for all lengths in the set of results ( roughly 0..4096, some lengths are skipped by the benchmarks)

def load_set_js(fnames):
    mr = {}

    for fname in fnames:
        r=load(fname);
        for length in r:
#each file's results are averaged for each tested length
            mean = statistics.mean(r[length])
            if length not in mr:
               mr[length]=[]
#and then added to a new dictionary that again uses the length as key
            mr[length].append(mean)

    return mr

def exp2():
    i = 1
    while sys.argv[i] != '--':
        i += 1

    print('loading first set', sys.argv[1:i-1])
    set1 = load_set_js(sys.argv[1:i-1])
    print()
    print('loading second set', sys.argv[i+1:])
    set2 = load_set_js(sys.argv[i+1:])
    print()

    print(" {:>8s} | {:>14} | {:>14} | {:>8}".format(
        label, "before (ns)", "after (ns)", "delta"))
    for length in sorted(set1):
#average over all runs of the test
        time1_ns = statistics.mean(set1[length])
        time2_ns = statistics.mean(set2[length])
        delta = time2_ns - time1_ns
        if time1_ns == 0:
            delta = 0
        else:
            delta = (delta / time1_ns) * 100.0
        print(" {:8} | {:14.2f} | {:14.2f} | {:7.2f}%".format(length, time1_ns, time2_ns, delta))

exp2()

